package org.netkernel.lang.dpml.ast.impl;


import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.endpoint.DPMLUtils;
import org.netkernel.lang.dpml.endpoint.RuntimeState;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.util.PairList;
import java.util.*;

public class AugmentedClosure extends VariableImpl implements IValueVariable
{
	private String mClosure;
	private String mVerb;
	private List mArguments;
	
	public AugmentedClosure(String aAssignment, IScope aScope, ILocation aLocation, String aClosure, String aVerb, List aArguments)
	{	super(aAssignment,aScope,aLocation);
		mClosure=aClosure;
		mVerb=aVerb;
		mArguments=aArguments;
	}
	
	public IValue getValue(INKFRequestContext aContext, RuntimeState aState, Class aRepresentationClass) throws NKFException
	{	IVariable toCall= getScope().resolveAssignment(mClosure,aState);
		if (toCall==null)
		{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_RUN_NOT_FOUND",null,null,mClosure);
		}
		if (!(toCall instanceof IValueVariable))
		{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_RUN_IDENTIFIER_NOT_IMPL",null,null,mClosure);
		}
		if ( (mArguments.size()>0 || !mVerb.equals("SOURCE")) && (!(toCall instanceof Closure) && !(toCall instanceof Sequence) ))
		{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_RUN_NOT_CLOSURE",null,null,mClosure);
		}
		
		//Sort out arguments		
		PairList args=new PairList(1+mArguments.size());
		args.put("verb", DPMLUtils.createArgumentLiteralValue("verb", mVerb, null, getLocation()));
		for (int i=0; i<mArguments.size(); i++)
		{	IArgument arg=(IArgument)mArguments.get(i);
			IVariable var = DPMLUtils.processArgument(arg,aContext,aState,true);
			args.put(arg.getName(),var);
		}
		RuntimeState rs=new RuntimeState(aState,args);
		
		IValue result= ((IValueVariable)toCall).getValue(aContext,rs,aRepresentationClass);
		return result;
	}
	
	public String toString()
	{	return "Closure["+mClosure+"]";
	}

}
