package org.netkernel.lang.dpml.ast.impl;

import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.endpoint.DPMLRuntime;
import org.netkernel.lang.dpml.endpoint.DPMLUtils;
import org.netkernel.lang.dpml.endpoint.RuntimeState;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFRequestImpl;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponse;
import org.netkernel.scheduler.ResolutionFailureException;
import org.netkernel.urii.IIdentifier;

import java.net.URI;
import java.net.URISyntaxException;


public class Dereference extends VariableImpl implements IIdentifierVariable
{
	private final IVariable mSource;
	
	public Dereference(String aName, IScope aScope, ILocation aLocation, IVariable aSource)
	{	super(aName,aScope,aLocation);
		mSource=aSource;
	}
	
	public String getIdentifier(INKFRequestContext aContext, RuntimeState aState) throws NKFException
	{	
		String result;
		IVariable var= DPMLUtils.dereference(mSource,aState,aContext);
		try
		{
			if (var instanceof IIdentifierVariable)
			{	
				IValue v=DPMLUtils.requestIdentifier((IIdentifierVariable)var, aContext, aState, String.class);
				result=v.getRepresentation().toString();
			}
			else
			{	IValue value = ((IValueVariable)var).getValue(aContext,aState,String.class);
				Object rep=value.getRepresentation();
				if (rep instanceof String)
				{	result=(String)rep;
				}
				else
				{	rep = (String) DPMLUtils.ensureClassOfValue(value,String.class,aContext);
					result=rep.toString();
				}
			}
		}
		catch (NKFException e)
		{	Throwable cause=e.getDeepestCause();
			if (cause instanceof ResolutionFailureException)
			{	ResolutionFailureException rfe=(ResolutionFailureException)cause;
				if (rfe.getRequest().getVerb()==IRequest.VERB_TRANSREPT && rfe.getRequest().getRepresentationClass()==IIdentifier.class)
				{	
					IResponse primary=(IResponse)rfe.getRequest().getValue(IRequest.TYPE_SOURCED_RESPONSE);
					String primaryClass=primary!=null?primary.getRepresentation().getClass().getName():"<null>";
					throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_BAD_DEREFERENCE", DPMLUtils.formatLocationMessage(var,aContext),null,primaryClass);
				}
			}
			throw e; //rethrow
		}
		return result;
	}
	
	public boolean isAbsolute()
	{	return true;
	}


}
