package org.netkernel.lang.dpml.ast.impl;

import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.endpoint.CompilerScope;
import org.netkernel.lang.dpml.endpoint.DPMLUtils;
import org.netkernel.lang.dpml.endpoint.RuntimeState;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.MetaImpl;

import java.util.*;

public class Sequence extends VariableImpl implements IScope, IValueVariable
{
	private List<IVariable> mAssignments;
	private CompilerScope mCompilerScope;
	private LiteralValue mNullResponse;
	
	public Sequence(String aName, IScope aScope, ILocation aLocation, CompilerScope aCompilerScope)
	{	super(aName,aScope,aLocation);
		mAssignments=new ArrayList<IVariable>();
		mCompilerScope=aCompilerScope;
		mNullResponse=new LiteralValue(ANON_PREFIX, aScope, aLocation, null, new MetaImpl(ExpiryFactory.getAlwaysExpired()));
	}

	public void addAssignment(IVariable aVariable, boolean aReplace) throws NKFException
	{	mAssignments.add(aVariable);
	}
	
	public void addImport(IScope aImport, int aPrecedence)
	{
	}

	public void setResponse(IVariable response)
	{
	}

	public IScope getParent()
	{	return getScope();
	}

	public Collection<IVariable> getVariables()
	{	return mAssignments;
	}

	
	public IValue getValue(INKFRequestContext aContext, RuntimeState aState, Class aRepresentationClass) throws NKFException
	{
		
		RuntimeState state;
		if (getScope() instanceof Sequence)
		{	state=aState;
		}
		else
		{	state=new RuntimeState(aState,this);
		}
		
		//RuntimeState state =new RuntimeState(aState,this);
		
		//run through assignments and process them
		for (IVariable v : mAssignments)
		{	String name=v.getName();
			IVariable v2 = DPMLUtils.dereference(v,state,aContext);
			IVariable assignment=null;
			if (v2 instanceof IValueVariable)
			{	IValueVariable vv = (IValueVariable)v2;
				IValue value=vv.getValue(aContext, state, Object.class);
				if (name!=null)
				{	assignment=new LiteralValue(name,null,getLocation(),value.getRepresentation(),value.getMeta());
				}
			}
			else
			{	assignment=v2;
			}
			
			if (name!=null && assignment!=null && !v.isAnonymous())
			{	state.putAssignment(this, name, assignment);
			}
		}
		
		//determine and evaluate response
		IVariable resp = state.getAssignment(this, Closure.RESPONSE);
		IValue result;
		if (resp instanceof IIdentifierVariable)
		{	result=DPMLUtils.requestIdentifier((IIdentifierVariable)resp,aContext,aState,aRepresentationClass);
		}
		else if (resp instanceof IValueVariable)
		{	result= ((IValueVariable)resp).getValue(aContext,aState,aRepresentationClass);
		}
		else
		{	result=mNullResponse.getValue(aContext, aState, aRepresentationClass);
		}		
		return result;		
	}
	
	public IVariable resolveAssignment(String aName, RuntimeState aState)
	{	
		IVariable result= null;
		RuntimeState state=aState;
		while (state!=null && state.getAssignmentsScope()!=null && !(state.getAssignmentsScope() instanceof Sequence))
		{	state=state.getParent();
		}
		while (state!=null && state.getAssignmentsScope()!=null && state.getAssignmentsScope() instanceof Sequence)
		{
			result=state.getAssignment(this, aName);
			if (result==null)
			{	state=state.getParent();
			}
			else
			{	break;
			}
		}
		if (result==null)
		{	IScope parent=getParent();
			if (parent!=null)
			{	result=parent.resolveAssignment(aName, state);
			}
		}
		return result;
	}

	public CompilerScope getCompilerScope()
	{	return mCompilerScope;
	}

}
