package org.netkernel.lang.dpml.endpoint;

import java.util.*;

public class CompilerScope
{
	private final CompilerScope mParent;
	private List<RequestAlias> mAliases;
	
	public CompilerScope(CompilerScope aParent)
	{	mParent=aParent;
		mAliases=null;
	}
	
	public void declareAlias(RequestAlias aAlias)
	{	if (mAliases==null)
		{	mAliases=new ArrayList<RequestAlias>();
		}
		mAliases.add(aAlias);
	}
	
	public CompilerScope getParent()
	{	return mParent;
	}
	
	public List<RequestAlias> getAliases()
	{	List<RequestAlias> result;
		if (mAliases!=null)
		{	result=Collections.unmodifiableList(mAliases);
		}
		else
		{	result = Collections.EMPTY_LIST;
		}
		return result;
	}
	
	public RequestAlias lookupTag(String aTagName)
	{	RequestAlias result=null;
		CompilerScope scope=this;
		while(scope!=null)
		{	result=scope.innerLookupTag(aTagName);
			if (result!=null)
			{	break;
			}
			scope=scope.getParent();
		}
		return result;
	}
	
	private RequestAlias innerLookupTag(String aTagName)
	{	RequestAlias result=null;
		if (mAliases!=null)
		{	for (RequestAlias alias: mAliases)
			{	if (alias.getTag().equals(aTagName))
				{	result=alias;
					break;
				}
			}
		}
		return result;
	}
	
	public static class RequestAlias
	{
		private final String mTag;
		private final String mIdentifier;
		
		public RequestAlias(String aTag, String aIdentifier)
		{	mTag=aTag;
			mIdentifier=aIdentifier;
		}
		
		public String getTag()
		{	return mTag;
		}
		
		public String getIdentifier()
		{	return mIdentifier;
		}		
	}
}
