package org.netkernel.lang.dpml.endpoint;

import java.util.*;

import org.netkernel.lang.dpml.ast.IArgument;
import org.netkernel.lang.dpml.ast.ILocation;
import org.netkernel.lang.dpml.ast.IScope;
import org.netkernel.lang.dpml.ast.IVariable;
import org.netkernel.lang.dpml.ast.impl.LiteralIndentifier;
import org.netkernel.lang.dpml.ast.impl.LiteralValue;
import org.netkernel.lang.dpml.ast.impl.LocationImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.urii.ParsedIdentifierImpl;
import org.netkernel.layer0.util.CheapMap;
import org.netkernel.layer0.util.PairList;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponseMeta;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.request.impl.RequestResponseFieldsImpl;

/** Table to store assignments in within the namespace of a scope
 * @author tab
 *
 */
public class RuntimeState
{
	private final RuntimeState mParent;
	private final String mScriptIdentifier; 
	private final PairList mArguments;
	private final IRequestScopeLevel mArgumentScope;
	private final IScope mAssignmentsScope;
	private CheapMap mAssignments;
	
	/** Initialise RuntimeState for new Runtime invocation **/
	public RuntimeState(INKFRequestContext aContext, String aScriptIdentifier)
	{	mParent=null;
		mAssignmentsScope=null;
		mScriptIdentifier=aScriptIdentifier;
		INKFRequestReadOnly thisReq=aContext.getThisRequest();
		PairList arguments=new PairList(1+thisReq.getArgumentCount());
		ILocation loc=new LocationImpl(aScriptIdentifier, -1, -1);
		arguments.put(IArgument.ARG_VERB, DPMLUtils.createArgumentLiteralValue(IArgument.ARG_VERB, RequestFactory.verbString(aContext.getThisRequest().getVerb()), null, loc));
		
		//arg:arguments
		HDSBuilder argsBuilder=new HDSBuilder();
		argsBuilder.pushNode(IArgument.ARG_ARGUMENTS);
		for (int i=0; i<thisReq.getArgumentCount(); i++)
		{	String name=thisReq.getArgumentName(i);
			if (name.equals("operator") || name.equals("response") || name.equals(ParsedIdentifierImpl.ARG_ACTIVE_SCHEME) || name.equals(ParsedIdentifierImpl.ARG_ACTIVE_TYPE)) continue;
			String reference=thisReq.getArgumentValue(i);
			IVariable var = new LiteralIndentifier(name,null,loc,reference,true);
			argsBuilder.addNode(name,reference);
		}
		IVariable argsVar=DPMLUtils.createArgumentLiteralValue(IArgument.ARG_ARGUMENTS,argsBuilder.getRoot(),null,loc);
		arguments.put(IArgument.ARG_ARGUMENTS,argsVar);		
				
		//all the other passed in arguments
		for (int i=0; i<thisReq.getArgumentCount(); i++)
		{	String name=thisReq.getArgumentName(i);
			if (name.equals("operator") || name.equals("response") || name.equals(ParsedIdentifierImpl.ARG_ACTIVE_SCHEME) || name.equals(ParsedIdentifierImpl.ARG_ACTIVE_TYPE)) continue;
			String value=thisReq.getArgumentValue(i);
			arguments.put(name, value);
		}
		//primary argument
		INKFResponseReadOnly primary=thisReq.getPrimaryAsResponse();
		if (primary!=null)
		{	IResponseMeta meta=((NKFResponseReadOnlyImpl)primary).getMeta();
			IVariable v= DPMLUtils.createArgumentLiteralValue(IArgument.ARG_PRIMARY, primary.getRepresentation(), meta, loc);
			arguments.put(IArgument.ARG_PRIMARY, v);
		}
		
		mArgumentScope=aContext.getKernelContext().getRequestScope();
		mArguments=arguments;		
	}

	/** Initialise RuntimeState for augmented closure **/
	public RuntimeState(RuntimeState aParent, PairList aArguments /*, String aVerb, ILocation aLocation*/)
	{	mParent=aParent;
		mAssignmentsScope=null;
		mScriptIdentifier=mParent.mScriptIdentifier;
		mArgumentScope=null;
		mArguments=aArguments;
	}
	
	/** Initialise RuntimeState for nested closure or sequence **/
	public RuntimeState(RuntimeState aParent,IScope aAssignmentsScope)
	{	mParent=aParent;
		mAssignmentsScope=aAssignmentsScope;
		mScriptIdentifier=mParent.mScriptIdentifier;
		mArguments=null;
		mArgumentScope=null;
	}
	
	public RuntimeState getParent()
	{	return mParent;
	}
	
	public IScope getAssignmentsScope()
	{	return mAssignmentsScope;
	}
	
	public IRequestScopeLevel getArgumentScope()
	{	return mArgumentScope;
	}
	
	public Object getArgument(String aName)
	{	return mArguments!=null?mArguments.get(aName):null;
	}
	
	public PairList getArguments()
	{	return mArguments;
	}
	
	public String getScriptIdentifier()
	{	return mScriptIdentifier;
	}
	
	public void putAssignment(IScope aScope, String aName, IVariable aVar)
	{	if (aName!=null)
		{	if (mAssignments==null)
			{	mAssignments=new CheapMap(8);
			}
			mAssignments.put(aName, aVar);
		}
	}
	
	public IVariable getAssignment(IScope aScope, String aName)
	{	IVariable result= mAssignments!=null?(IVariable)mAssignments.get(aName):null;
		return result;
	}
}
